@@ -1,4 +1,4 @@
-/* $Id: menueditor-main-window.c 23114 2006-09-09 13:35:41Z pollux $ */
+/* $Id: menueditor-main-window.c 26914 2008-05-01 07:04:05Z kelnos $ */
 /*
  * Copyright (c) 2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -48,7 +48,7 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/libxfcegui4.h>
 
-#include <xfdesktop-common.h>
+#include <desktop-menu-utils.h>
 
 #include "menueditor-add-dialog.h"
 #include "menueditor-add-external-dialog.h"
@@ -56,7 +56,7 @@
 #include "menueditor-edit-external-dialog.h"
 #include "menueditor-main-window.h"
 
-#include "../modules/menu/dummy_icon.h"
+#include "dummy_icon.h"
 
 #define MENUEDITOR_MAIN_WINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), MENUEDITOR_TYPE_MAIN_WINDOW, MenuEditorMainWindowPrivate))
 
