@@ -3,7 +3,7 @@
  *  Copyright (C) 2002-2003 Jasper Huijsmans (huysmans@users.sourceforge.net)
  *                     2003 Biju Chacko (botsie@users.sourceforge.net)
  *                     2004 Danny Milosavljevic <danny.milo@gmx.net>
- *                2004-2007 Brian Tarricone <bjt23@cornell.edu>
+ *                2004-2008 Brian Tarricone <bjt23@cornell.edu>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -77,14 +77,11 @@ typedef struct
     GList *menu_item_cache;
 	
     gchar *filename;  /* file the menu is currently using */
-    gboolean using_default_menu;
     
     gboolean use_menu_icons;  /* show menu icons? */
 	
     gint idle_id;  /* source id for idled generation */
     
-    gboolean modified;
-    
 #ifdef HAVE_THUNAR_VFS
     GList *monitors;
 #endif
@@ -95,8 +92,8 @@ static void _xfce_desktop_menu_free_menudata(XfceDesktopMenu *desktop_menu);
 static gint _xfce_desktop_menu_icon_size = 24;
 static GtkIconTheme *_deskmenu_icon_theme = NULL;
 
-static gboolean _generate_menu(XfceDesktopMenu *desktop_menu,
-                               gboolean deferred);
+static gboolean _generate_menu_idled(gpointer data);
+static gboolean _generate_menu(XfceDesktopMenu *desktop_menu);
 static void desktop_menu_add_items(XfceDesktopMenu *desktop_menu,
                                    XfceMenu *xfce_menu,
                                    GtkWidget *menu,
@@ -105,7 +102,17 @@ static void desktop_menu_add_items(XfceDesktopMenu *desktop_menu,
 static void
 itheme_changed_cb(GtkIconTheme *itheme, gpointer user_data)
 {
-    _generate_menu((XfceDesktopMenu *)user_data, FALSE);
+    XfceDesktopMenu *desktop_menu = user_data;
+
+    /* this fixes bugs 3615 and 4342.  if both the .desktop files
+     * and icon theme change very quickly after each other, we'll
+     * get a crash when the icon theme gets regenerated when calling
+     * gtk_icon_theme_lookup_icon(), which triggers a recursive regen.
+     * so we'll idle the regen, and make sure we don't enter it
+     * recursively.  same deal for _something_changed(). */
+
+    if(!desktop_menu->idle_id)
+        desktop_menu->idle_id = g_idle_add(_generate_menu_idled, desktop_menu);
 }
 
 #ifdef HAVE_THUNAR_VFS
@@ -130,7 +137,8 @@ desktop_menu_something_changed(ThunarVfsMonitor *monitor,
 #endif
     
     xfce_menu_item_cache_invalidate(cache);
-    _generate_menu(desktop_menu, FALSE);
+    if(!desktop_menu->idle_id)
+        desktop_menu->idle_id = g_idle_add(_generate_menu_idled, desktop_menu);
 }
 
 #if 0
@@ -232,6 +240,7 @@ desktop_menu_add_items(XfceDesktopMenu *desktop_menu,
     }
     for(l = items; l; l = l->next) {
         if(XFCE_IS_MENU(l->data)) {
+            GList *tmpl;
             xfce_submenu = l->data;
             xfce_directory = xfce_menu_get_directory(xfce_submenu);
             icon_name = NULL;
@@ -273,8 +282,10 @@ desktop_menu_add_items(XfceDesktopMenu *desktop_menu,
             
             /* we have to check emptiness down here instead of at the top of the
              * loop because there may be further submenus that are empty */
-            if(!gtk_container_get_children(GTK_CONTAINER(submenu)))
+            if(!(tmpl = gtk_container_get_children(GTK_CONTAINER(submenu))))
                 gtk_widget_destroy(mi);
+            else
+                g_list_free(tmpl);
         } else if(XFCE_IS_MENU_SEPARATOR(l->data)) {
             mi = gtk_separator_menu_item_new();
             gtk_widget_show(mi);
@@ -321,8 +332,7 @@ desktop_menu_add_items(XfceDesktopMenu *desktop_menu,
 }
 
 static gboolean
-_generate_menu(XfceDesktopMenu *desktop_menu,
-               gboolean force)
+_generate_menu(XfceDesktopMenu *desktop_menu)
 {
     gboolean ret = TRUE;
     XfceKiosk *kiosk;
@@ -360,7 +370,7 @@ _generate_menu(XfceDesktopMenu *desktop_menu,
         desktop_menu_add_items(desktop_menu, desktop_menu->xfce_menu,
                                NULL, &desktop_menu->menu_item_cache);
     }
-    
+
     return ret;
 }
 
@@ -383,12 +393,13 @@ _xfce_desktop_menu_free_menudata(XfceDesktopMenu *desktop_menu)
 }
 
 static gboolean
-_generate_menu_initial(gpointer data) {
+_generate_menu_idled(gpointer data)
+{
     XfceDesktopMenu *desktop_menu = data;
     
     g_return_val_if_fail(data != NULL, FALSE);
     
-    _generate_menu(desktop_menu, FALSE);
+    _generate_menu(desktop_menu);
     desktop_menu->idle_id = 0;
     
     return FALSE;
@@ -423,10 +434,8 @@ xfce_desktop_menu_new_impl(const gchar *menu_file,
     
     if(menu_file)
         desktop_menu->filename = g_strdup(menu_file);
-    else {
+    else
         desktop_menu->filename = xfce_desktop_get_menufile();
-        desktop_menu->using_default_menu = TRUE;
-    }
     
 #ifdef HAVE_THUNAR_VFS
     thunar_vfs_init();
@@ -434,9 +443,9 @@ xfce_desktop_menu_new_impl(const gchar *menu_file,
 #endif
     
     if(deferred)
-        desktop_menu->idle_id = g_idle_add(_generate_menu_initial, desktop_menu);
+        desktop_menu->idle_id = g_idle_add(_generate_menu_idled, desktop_menu);
     else {
-        if(!_generate_menu(desktop_menu, FALSE)) {
+        if(!_generate_menu(desktop_menu)) {
 #ifdef HAVE_THUNAR_VFS
             xfce_menu_monitor_set_vtable(NULL, NULL);
 #endif
@@ -458,7 +467,11 @@ xfce_desktop_menu_populate_menu_impl(XfceDesktopMenu *desktop_menu,
     g_return_if_fail(desktop_menu && menu);
     
     if(!desktop_menu->xfce_menu) {
-        _generate_menu(desktop_menu, FALSE);
+        if(desktop_menu->idle_id) {
+            g_source_remove(desktop_menu->idle_id);
+            desktop_menu->idle_id = 0;
+        }
+        _generate_menu(desktop_menu);
         if(!desktop_menu->xfce_menu)
             return;
     }
@@ -529,7 +542,12 @@ xfce_desktop_menu_force_regen_impl(XfceDesktopMenu *desktop_menu)
     TRACE("dummy");
     g_return_if_fail(desktop_menu != NULL);
     
-    _generate_menu(desktop_menu, TRUE);
+    if(desktop_menu->idle_id) {
+        g_source_remove(desktop_menu->idle_id);
+        desktop_menu->idle_id = 0;
+    }
+
+    _generate_menu(desktop_menu);
 }
 
 G_MODULE_EXPORT void
@@ -540,7 +558,11 @@ xfce_desktop_menu_set_show_icons_impl(XfceDesktopMenu *desktop_menu,
     
     if(desktop_menu->use_menu_icons != show_icons) {
         desktop_menu->use_menu_icons = show_icons;
-        _generate_menu(desktop_menu, TRUE);
+        if(desktop_menu->idle_id) {
+            g_source_remove(desktop_menu->idle_id);
+            desktop_menu->idle_id = 0;
+        }
+        _generate_menu(desktop_menu);
     }
 }
 
