@@ -317,6 +317,14 @@ xfce_backdrop_init(XfceBackdrop *backdrop)
     backdrop->priv = G_TYPE_INSTANCE_GET_PRIVATE(backdrop, XFCE_TYPE_BACKDROP,
                                                  XfceBackdropPriv);
     backdrop->priv->show_image = TRUE;
+
+    /* color defaults */
+    backdrop->priv->color1.red = 0x0f0f;
+    backdrop->priv->color1.green = 0x1717;
+    backdrop->priv->color1.blue = 0x2121;
+    backdrop->priv->color2.red = 0x0f0f;
+    backdrop->priv->color2.green = 0x1717;
+    backdrop->priv->color2.blue = 0x2121;
 }
 
 static void
@@ -914,7 +922,6 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
     if(backdrop->priv->brightness != 0)
         final_image = adjust_brightness(final_image, backdrop->priv->brightness);
 
-    g_debug("saturation is %.4f", backdrop->priv->saturation);
     if(backdrop->priv->saturation > 1.01 || backdrop->priv->saturation < 0.99) {
         gdk_pixbuf_saturate_and_pixelate(final_image, final_image,
                                          (gfloat)backdrop->priv->saturation,
